<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/
class BOOKMARKS_CMP_Notify extends OW_Component
{
    private $user;
    
    public function __construct( $userId, $idList )
    {
        parent::__construct();
        
        if ( !empty($userId) && !empty($idList) )
        {
            $this->user = BOL_UserService::getInstance()->findUserById($userId);
            
            $userService = BOL_UserService::getInstance();
            $avatars = BOL_AvatarService::getInstance()->getAvatarsUrlList($idList, 2);
            $sexValue = array();
            $list = array();
            
            foreach ( BOL_QuestionValueDao::getInstance()->findQuestionValues('sex') as $sexDto )
            {
                $sexValue[$sexDto->value] = BOL_QuestionService::getInstance()->getQuestionValueLang('sex', $sexDto->value);
            }

            $userData = BOL_QuestionService::getInstance()->getQuestionData($idList, array('sex', 'birthdate', 'googlemap_location'));
            
            foreach ( $idList as $userId )
            {
                $list[$userId]['userUrl'] = $userService->getUserUrl($userId);
                $list[$userId]['displayName'] = $userService->getDisplayName($userId);
                $list[$userId]['avatarUrl'] = $avatars[$userId];
                $list[$userId]['activity'] = UTIL_DateTime::formatDate(BOL_UserService::getInstance()->findUserById($userId)->getActivityStamp());

                if ( !empty($userData[$userId]['birthdate']) )
                {
                    $date = UTIL_DateTime::parseDate($userData[$userId]['birthdate'], UTIL_DateTime::MYSQL_DATETIME_DATE_FORMAT);

                    $list[$userId]['age'] = UTIL_DateTime::getAge($date['year'], $date['month'], $date['day']);
                }

                if ( !empty($userData[$userId]['sex']) )
                {
                    $list[$userId]['sex'] = $sexValue[$userData[$userId]['sex']];
                }
                
                if ( !empty($userData[$userId]['googlemap_location']) )
                {
                    $list[$userId]['googlemap_location'] = $userData[$userId]['googlemap_location']['address'];
                }
            }
            
            $this->assign('userName', BOL_UserService::getInstance()->getDisplayName($this->user->id));
            $this->assign('list', $list);
        }
        else
        {
            $this->setVisible(FALSE);
        }
    }

    private function getHtml()
    {
        $this->setTemplate(OW::getPluginManager()->getPlugin('bookmarks')->getCmpViewDir() . 'notify_html.html');

        return parent::render();
    }

    public function sendNotification()
    {
        if ( empty($this->user) )
        {
            return;
        }
        
        $content = $this->getHtml();
        
        $mail = OW::getMailer()->createMail()
            ->addRecipientEmail($this->user->email)
            ->setSubject(OW::getLanguage()->text('bookmarks', 'email_notify_subject'))
            ->setHtmlContent($content)
            ->setTextContent($content);

        OW::getMailer()->send($mail);
    }
}
