<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/

class BOOKMARKS_MCLASS_EventHandler
{
    private static $classInstance;

    /**
     * 
     * @return BOOKMARKS_MCLASS_EventHandler
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }
    
    private function __construct()
    {

    }
    
    public function onActionToolbarAddUserBookmarkTool( BASE_CLASS_EventCollector $event )
    {
        $params = $event->getParams();

        if ( !OW::getUser()->isAuthenticated() || $params['userId'] == OW::getUser()->getId() )
        {
            return;
        }
        
        $uniqId = uniqid('bookmarks-');
        $languages = OW::getLanguage();
        $markLabel = $languages->text('bookmarks', 'mark_toolbar_label');
        $unMarkLabel = $languages->text('bookmarks', 'unmark_toolbar_label');
        
        $isMarked = BOOKMARKS_BOL_Service::getInstance()->isMarked(OW::getUser()->getId(), $params['userId']);
        
        $label = $isMarked ? $unMarkLabel : $markLabel;
        
        $event->add(array(
            BASE_CMP_ProfileActionToolbar::DATA_KEY_ITEM_KEY => "bookmark.action",
            BASE_CMP_ProfileActionToolbar::DATA_KEY_LINK_ID => $uniqId,
            BASE_CMP_ProfileActionToolbar::DATA_KEY_LABEL => $label,
            BASE_CMP_ProfileActionToolbar::DATA_KEY_LINK_ORDER => 1,
            'group' => 'addition',
            BASE_CMP_ProfileActionToolbar::DATA_KEY_LINK_ATTRIBUTES => array("data-command" => $isMarked),
            BASE_CMP_ProfileActionToolbar::DATA_KEY_LINK_HREF => 'javascript://'
        ));
        
        OW::getDocument()->addScriptDeclarationBeforeIncludes(
            UTIL_JsGenerator::composeJsString(';window.bookmarksActionUrl = {$url};',
                array(
                    'url' => OW::getRouter()->urlForRoute('bookmarks.mark_state')
                )
            )
        );
        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('bookmarks')->getStaticJsUrl() . 'bookmarks.js');
        
        OW::getDocument()->addOnloadScript(
            UTIL_JsGenerator::composeJsString(';var bookmarkBtn = $("#" + {$id}).on("click", function(event)
                {
                    BOOKMARKS.markState({$userId}, function( data, textStatus, jqXHR )
                    {
                        if ( data.mark === true )
                        {
                            OWM.info(\'<div class="clearfix bookmarks_wrap"><span>\' + {$markedNotifyMessage} + \'</span><span class="ow_left boomarks_ic_wrap bookmarks_ic_bookmark_white"></span></div>\');
                            bookmarkBtn.find("span").text({$unMarkLabel});
                        }
                        else
                        {
                            OWM.info(\'<div class="clearfix bookmarks_wrap"><span>\' + {$unMarkedNotifyMessage} + \'</span><span class="ow_left boomarks_ic_wrap bookmarks_ic_bookmark_white"></span></div>\');
                            bookmarkBtn.find("span").text({$markLabel});
                        }
                    });
                });',
                array(
                    'id' => $uniqId,
                    'userId' => $params['userId'],
                    'markedNotifyMessage' => $languages->text('bookmarks', 'marked_notify_message', array('bookmarksListURL' => OW::getRouter()->urlForRoute('bookmarks.list', array('category' => BOOKMARKS_BOL_Service::LIST_LATEST)))),
                    'unMarkedNotifyMessage' => $languages->text('bookmarks', 'unmarked_notify_message', array('bookmarksListURL' => OW::getRouter()->urlForRoute('bookmarks.list', array('category' => BOOKMARKS_BOL_Service::LIST_LATEST)))),
                    'unMarkLabel' => $unMarkLabel,
                    'markLabel' => $markLabel
                )
            )
        );
    }
    
    public function onBeforeDecoratorRender( BASE_CLASS_PropertyEvent $event )
    {
        $prop = $event->getProperties();

        if ( OW::getUser()->isAuthenticated() && !empty($prop['name']) && $prop['name'] == 'avatar_item' && !empty($prop['data']['userId']) )
        {
            $isMarked = BOOKMARKS_BOL_Service::getInstance()->isMarked(OW::getUser()->getId(), $prop['data']['userId']);
            
            if ( $isMarked )
            {
                $event->setProperty('isMarked', $isMarked);
            }
        }
    }
    
    public function init() {
        BOOKMARKS_CLASS_EventHandler::getInstance()->genericInit();
        
        OW::getEventManager()->bind(BASE_MCMP_ProfileActionToolbar::EVENT_NAME, array($this, 'onActionToolbarAddUserBookmarkTool'));
        OW::getEventManager()->bind('base.before_decorator', array($this, 'onBeforeDecoratorRender'));        
    }
}
