<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

abstract class GOOGLELOCATION_CMP_MapWidget extends BASE_CLASS_Widget
{    
    const MAX_USERS_COUNT = 2000;
    
    protected $map = null;
    protected $mapHeight = null;    
    protected $idList = array();   
    
    public function __construct( BASE_CLASS_WidgetParameter $params )
    {
        parent::__construct();
        
        $IdList = $this->assignList( $params );
        
        if ( empty($IdList) && !$params->customizeMode )
        {
            $this->setVisible(false);
            return;
        }

        $this->mapHeight = isset($params->customParamList['map_height']) ? (int) $params->customParamList['map_height'] : 350;
        $this->renderMap($IdList, $params);
        
        $this->setTemplate(OW::getPluginManager()->getPlugin('googlelocation')->getCmpViewDir().'map_widget.html');
    }
    
    abstract protected function assignList( BASE_CLASS_WidgetParameter $params );

    protected function getMap( BASE_CLASS_WidgetParameter $params )            
    {        
        return GOOGLELOCATION_BOL_LocationService::getInstance()->getMapComponent();
    }
    
    protected function renderMap($IdList, BASE_CLASS_WidgetParameter $params)
    {        
        $event = new OW_Event( 'googlelocation.get_map_component', array( 'userIdList' => $IdList, 'backUri' => OW::getRouter()->getUri() ) );
        OW::getEventManager()->trigger($event);
        /* @var $map GOOGLELOCATION_CMP_Map */
        $map = $event->getData();
        $map->setHeight($this->mapHeight . 'px');
        
        if ( !empty($params->customParamList['map_display_search']) )
        {
            $map->displaySearchInput(true);
        }
        
        OW::getEventManager()->trigger(new OW_Event('googlelocation.add_js_lib'));   

        $this->addComponent("map", $map);
    }
    
    public static function getSettingList()
    {
        $settingList = array();
        
        $settingList['map_height'] = array(
            'presentation' => self::PRESENTATION_NUMBER,
            'label' => OW_Language::getInstance()->text('googlelocation', 'widget_settings_map_height'),
            'value' => 350
        );
        
        $settingList['map_display_search'] = array(
            'presentation' => self::PRESENTATION_CHECKBOX,
            'label' => OW_Language::getInstance()->text('googlelocation', 'widget_settings_display_search'),
            'value' => false
        );

        return $settingList;
    }

    public static function getStandardSettingValueList()
    {
        return array(
            self::SETTING_SHOW_TITLE => true,
            self::SETTING_WRAP_IN_BOX => true,
            self::SETTING_TITLE => OW_Language::getInstance()->text('googlelocation', 'widget_map_title'),
            self::SETTING_ICON => self::ICON_BOOKMARK
        );
    }

    public static function getAccess()
    {
        return self::ACCESS_MEMBER;
    }
}
