<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class MATCHMAKING_BOL_QuestionMatchDao extends OW_BaseDao
{    
    /**
     * Singleton instance.
     *
     * @var MATCHMAKING_BOL_QuestionMatchDao
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return MATCHMAKING_BOL_QuestionMatchDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'MATCHMAKING_BOL_QuestionMatch';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'matchmaking_question_match';
    }

    /**
     * @return mixed|null|string
     */
    public function getMaxPercentValue()
    {
        $query = "SELECT SUM(`coefficient`) FROM " . $this->getTableName() . " WHERE 1";

        return $this->dbo->queryForColumn($query);
    }

    /**
     * @return array|mixed
     */
    public function findRequiredMatchFields()
    {
        $example = new OW_Example();
        $example->andFieldEqual('required', 1);

        return $this->findListByExample($example);
    }

    /**
     * @return array|mixed
     */
    public function findFieldsExceptRequired()
    {
        $example = new OW_Example();
//        $example->andFieldEqual('required', 0);

        return $this->findListByExample($example);
    }

    /**
     * @param $userId
     * @return mixed|null|string
     */
    public function findMatchCount( $userId )
    {
        $params = $this->getQueryParamsForCountMatches($userId);

        if (empty($params))
        {
            return 0;
        }

        $query = $this->prepareQuerySelectCount($params);

        return $this->dbo->queryForColumn($query);
    }

    /**
     * @param $userId
     * @param $sortOrder
     * @param $first
     * @param $count
     * @return array
     */
    public function findUserIdMatchListByQuestionValues( $userId, $sortOrder, $first, $count )
    {
        $params = $this->getQueryParams($userId, $sortOrder);

        if (empty($params))
        {
            return array();
        }

        $query = $this->prepareQuerySelect($params);

        return $this->dbo->queryForList($query, array_merge(array('first' => $first, 'count' => $count)));
    }

    /**
     * @param $params
     * @return string
     */
    private function prepareQuerySelectCount( $params )
    {
        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter('user', "id", array(
            "method" => "MATCHMAKING_BOL_QuestionMatchDao::prepareQuerySelectCount"
        ));
        
        $sql = "SELECT COUNT( DISTINCT `user`.id ) 
            FROM {$this->getUsersTableNameWithLimits()} `user`
        " . $queryParts['join'] . $params['join'] . "

        WHERE " . $queryParts['where'] ." AND `user`.`id`<>" . $params['userId'] . " " . $params['where'] . " AND  " . $params['compatibility'] . " > 0 ";

        return $sql;
    }

    /**
     * @param $params
     * @return string
     */
    private function prepareQuerySelect( $params )
    {
        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter('user', "id", array(
            "method" => "MATCHMAKING_BOL_QuestionMatchDao::prepareQuerySelect"
        ));
        
        $sql = " SELECT DISTINCT `user`.id, " . $params['compatibility'] . " as `compatibility` 
        
        FROM {$this->getUsersTableNameWithLimits()} `user`
            
        " . $queryParts['join'] . $params['join'] . "
            
        WHERE (" . $params['compatibility'] . ") <> 0 AND " . $queryParts['where'] ." AND `user`.`id`<>" . $params['userId'] . " " . $params['where'] . "
        ORDER BY " . $params['order'] . " `user`.`activityStamp` DESC
        LIMIT :first, :count ";
        
        return $sql;
    }

    /**
     * @param $params
     * @return string
     */
    private function prepareQuerySelectForUserIdList( $params )
    {
        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter('user', "id", array(
            "method" => "MATCHMAKING_BOL_QuestionMatchDao::prepareQuerySelect"
        ));
        
        $table = $this->getUsersTableNameWithLimits();
        
        if( !empty($params['userIdList']) )
        {
            $table = "`".BOL_UserDao::getInstance()->getTableName()."`";
        }
        
        $sql = " SELECT DISTINCT `user`.`id` as userId, " . $params['compatibility'] . " as `compatibility` 
            
        FROM {$table} `user`
            
        " . $queryParts['join'] . $params['join'] . "

        WHERE " . $queryParts['where'] ." AND `user`.`id`<>" . $params['userId'] . " " . $params['where'] . " 
        ORDER BY " . $params['order'] . " `user`.`activityStamp` DESC
        LIMIT :first, :count ";

        return $sql;
    }

    protected function getUsersTableNameWithLimits()
    {
        $usersTableName = "`".BOL_UserDao::getInstance()->getTableName()."`";
        
        if ( OW_SQL_LIMIT_USERS_COUNT > 0 )
        {
            $orderFieldname = BOL_UserDao::ACTIVITY_STAMP;
            $usersTableName = "( SELECT * FROM {$usersTableName} ORDER BY `{$orderFieldname}` DESC LIMIT " . OW_SQL_LIMIT_USERS_COUNT . " )";
        }
        
        return $usersTableName;
    }

        /**
     * @param $userId
     * @return array
     */
    private function getQueryParamsForCountMatches( $userId )
    {
        return $this->getQueryParamsForUserIdList($userId, array(),null, true);
    }
    
    /**
     * @param $userId
     * @param string $sortOrder
     * @return array
     */
    private function getQueryParams( $userId, $sortOrder = 'newest', $countMatches = false )
    {
        return $this->getQueryParamsForUserIdList($userId, array(),$sortOrder, $countMatches);
    }

    private function getQueryParamsForUserIdList( $userId, $userIdList, $sortOrder = 'newest', $countMatches = false )
    {
        $questionService = BOL_QuestionService::getInstance();
        $userService = BOL_UserService::getInstance();
        
        $matchFields = array();
        
        if ( $countMatches && empty($userIdList) )
        {
            $matchFields = $this->findRequiredMatchFields();
        }
        else
        {
            $matchFields = $this->findAll();
        }
        
        $prefix = 'qd';
        $counter = 0;
        $innerJoin = '';
        $leftJoin = '';
        $where = '';
        $compatibility = ' 0 +';
        $requiredCompatibility = ' 1 *';
        $location = null;
        $distance = MATCHMAKING_BOL_Service::getInstance()->getDistance($userId);
        
        if ( OW::getPluginManager()->isPluginActive('googlelocation') )
        {
            $location = GOOGLELOCATION_BOL_LocationService::getInstance()->findByUserId($userId);
            if ( $location ) {
                $location = get_object_vars($location);
            }
        }

        foreach ( $matchFields as $field )
        {
            $joinType = " INNER ";

            if ( !$countMatches )
            {
                $joinType = " LEFT ";
            }
            
            $question = $questionService->findQuestionByName($field->questionName);
            if (!$question)
            {
                continue;
            }
            
            // add location
            if ( $field->questionName == 'googlemap_location' && !empty($location) )
            {
                $locationSqlData = $this->getLocationSql($field, $location, $distance, $joinType);
                $leftJoin .= $locationSqlData['join'];
                $compatibility .= $locationSqlData['compatibility'];
                continue;
            }
            
            $matchQuestion = $questionService->findQuestionByName($field->matchQuestionName);
            if (!$matchQuestion)
            {
                continue;
            }
            $checkData = $questionService->getQuestionData(array($userId), array($field->matchQuestionName));

            if ( empty($checkData[$userId][$field->matchQuestionName]) )
            {
                if ($field->required)
                {
                    return array();
                }
            }

            $value1 = null;

            if (isset($checkData[$userId][$field->matchQuestionName]))
            {
                $value1 = $checkData[$userId][$field->matchQuestionName];
            }
            
            if ( !empty($value1) )
            {
                if ( $field->required )
                {
                    // calculate compatibility for required fields
                    $questionString = $this->prepareQuestionWhere($question, $value1, $prefix . $counter);

                    if ( !empty($questionString) )
                    {                             
                        $innerJoin .= $joinType . "JOIN `" . BOL_QuestionDataDao::getInstance()->getTableName() . "` `" . $prefix . $counter . "`
                        ON ( `user`.`id` = `" . $prefix . $counter . "`.`userId` AND `" . $prefix . $counter . "`.`questionName` = '" . $this->dbo->escapeString($question->name) . "' AND " . $questionString . " ) ";

                        $compatibility .= ' IF ( `' . $prefix . $counter . '`.`questionName` IS NOT NULL, ' . $field->coefficient . ', 0 ) +';
                        // if users don't match by required field than compatibility = 0
                        $requiredCompatibility .=  ' IF ( `' . $prefix . $counter . '`.`id` IS NOT NULL, 1, 0 ) *';

                        $counter++;
                    }

                    $checkData2 = $questionService->getQuestionData(array($userId), array($field->questionName));

                    if ( empty($checkData2[$userId][$field->questionName]) )
                    {
                        continue;
                    }
                    $value2 = $checkData2[$userId][$field->questionName];

                    $questionString = $this->prepareQuestionWhere($matchQuestion, $value2, $prefix . $counter);
                    if ( !empty($questionString) )
                    {
                        $innerJoin .= "
                            ".$joinType." JOIN `" . BOL_QuestionDataDao::getInstance()->getTableName() . "` `" . $prefix . $counter . "`
                        ON ( `user`.`id` = `" . $prefix . $counter . "`.`userId` AND `" . $prefix . $counter . "`.`questionName` = '" . $this->dbo->escapeString($matchQuestion->name) . "' AND " . $questionString . " ) ";

                        // if users don't match by required field than compatibility = 0
                        $requiredCompatibility .=  ' IF( `' . $prefix . $counter . '`.`questionName` IS NOT NULL, 1, 0 ) *';
                        
                        $counter++;
                    }
                }
                else
                {                    
                    // calculate compatibility for not required fields
                    $questionString = $this->prepareQuestionWhere($question, $value1, $prefix . $counter);

                    if ( !empty($questionString) )
                    {
                        $leftJoin .= "
                            ".$joinType." JOIN `" . BOL_QuestionDataDao::getInstance()->getTableName() . "` `" . $prefix . $counter
                            . "` ON ( `user`.`id` = `" . $prefix . $counter . "`.`userId` AND `" . $prefix . $counter . "`.`questionName` = '" . $this->dbo->escapeString($question->name) . "' AND " . $questionString . " ) ";

                        $compatibility .= ' IF( `' . $prefix . $counter . '`.`questionName` IS NOT NULL, ' . $field->coefficient . ', 0 ) +';
                        $counter++;
                    }
                }
            }
            else
            {
                $compatibility .= ' ' . $field->coefficient . ' +';
            }
        }

        /**/

        $order = "";
        if ( $countMatches == false )
        {
            if ( $sortOrder == 'newest' )
            {
                $order = ' `user`.`joinStamp` DESC, ';
            }
            else if ( $sortOrder == 'mail' )
            {
                $order = ' `user`.`joinStamp` DESC, ';

                $matchmaking_lastmatch_userid = BOL_PreferenceService::getInstance()->getPreferenceValue('matchmaking_lastmatch_userid', $userId );

                $where = ' AND `user`.`id` > '.$matchmaking_lastmatch_userid;
            }
            else if ( $sortOrder == 'compatible' )
            {
                //$order = '(' . substr($compatibility, 0, -1) . ') DESC , ';
                $order = ' `compatibility` DESC , ';
            }
        }
        
        if ( !empty($userIdList) )
        {
            $listOfUserIds = $this->dbo->mergeInClause($userIdList);
            $where .= ' AND `user`.`id` IN ( '.$listOfUserIds.' ) ';
        }

        $result = array(
            'userId' => $userId,
            'join' => $innerJoin . $leftJoin,
            'where' => $where,
            'order' => $order,
            'userIdList' => $userIdList,
            'compatibility' => ' (' . substr($compatibility, 0, -1) . ') * (' . substr($requiredCompatibility, 0, -1) . ') '
        );
        
        if ( !empty($userIdList) )
        {
            $result['userIdList'] = $userIdList;
        }
        
        return $result;
    }
    
    private function getLocationSql($field, $location, $distanse, $joinType = 'INNER')
    {
        $result = array(
            'join' => "",
            'where' => "",
            'compatibility' => ""
        );
        
        if ( OW::getPluginManager()->isPluginActive('googlelocation') )
        {
            $json = !empty($location['json']) ? json_decode($location['json'], true) : array();

            $countryCode = "";
            if ( !empty($json['address_components']) )
            {
                foreach ( $json['address_components'] as $component )
                {
                    if ( !empty($component['types']) && is_array($component['types']) && in_array('country', $component['types']) )
                    {
                        $countryCode = !empty($component['short_name']) ? $component['short_name'] : "";
                    }
                }
            }

            if ( !empty($distanse) && (float) $distanse > 0 )
            {
                $coord = GOOGLELOCATION_BOL_LocationService::getInstance()->getNewCoordinates($location['southWestLat'], $location['southWestLng'], 'sw', (float) $distanse);
                $location['southWestLat'] = $coord['lat'];
                $location['southWestLng'] = $coord['lng'];

                $coord = GOOGLELOCATION_BOL_LocationService::getInstance()->getNewCoordinates($location['northEastLat'], $location['northEastLng'], 'ne', (float) $distanse);
                $location['northEastLat'] = $coord['lat'];
                $location['northEastLng'] = $coord['lng'];
            }

            $sql = GOOGLELOCATION_BOL_LocationService::getInstance()->getSearchInnerJoinSql('user', $location['southWestLat'], $location['southWestLng'], $location['northEastLat'], $location['northEastLng'], $countryCode, $joinType);

            $result["join"] = " \n ".$sql;
            $result["compatibility"] = ' IF( `location`.`id` IS NOT NULL, ' . $field->coefficient . ', 0 ) +';
        }
        
        return $result;
    }
    
    /**
     * @param BOL_Question $question
     * @param $value
     * @param string $prefix
     * @return array|string
     */
    private function prepareQuestionWhere( BOL_Question $question, $value, $prefix = '' )
    {
        $result = '';
        $prefix = $this->dbo->escapeString($prefix);
        
        switch ( $question->presentation )
        {
            case BOL_QuestionService::QUESTION_PRESENTATION_URL :
            case BOL_QuestionService::QUESTION_PRESENTATION_TEXT :
            case BOL_QuestionService::QUESTION_PRESENTATION_TEXTAREA :
                if ( $question->base )
                {
                    $result = ' `user`.`' . $this->dbo->escapeString($question->name) . '` LIKE \'' . $this->dbo->escapeString($value) . '%\'';
                }
                else
                {
                    $result = " LCASE(`" . $prefix . "`.`textValue`) LIKE '" . $this->dbo->escapeString(strtolower($value)) . "%'";
                }
                break;

            case BOL_QuestionService::QUESTION_PRESENTATION_CHECKBOX :
                if ( $question->base )
                {
                    $result = ' `user`.`' . $this->dbo->escapeString($question->name) . '` = ' . (boolean) $value;
                }
                else
                {
                    $result = " `" . $prefix . "`.`intValue` = " . (boolean) $value;
                }
                break;

            case BOL_QuestionService::QUESTION_PRESENTATION_MULTICHECKBOX :
                if ( $question->base )
                {
                    $result = ' `user`.`' . $this->dbo->escapeString($question->name) . '` & ' . $value . ' ';
                }
                else
                {
                    $result = ' `' . $this->dbo->escapeString($prefix) . '`.`intValue` & ' . $value . ' ';
                }

                break;


            case BOL_QuestionService::QUESTION_PRESENTATION_RADIO :
            case BOL_QuestionService::QUESTION_PRESENTATION_SELECT :
                $questionValues = BOL_QuestionService::getInstance()->findQuestionValues($question->name);

                if ( !empty($questionValues) )
                {
                    $result = array();
                    foreach ( $questionValues as $val )
                    {
                        if ( (bool)($val->value & (int)$value) )
                        {
                            $result[] = $val->value;
                        }
                    }
                    $value = $result;
                }

                if ( isset($value) && is_array($value) && !empty($value) )
                {
                    if ( $question->base )
                    {
                        $result = ' `user`.`' . $this->dbo->escapeString($question->name) . '` IN ( ' . $this->dbo->mergeInClause($value) . ' ) ';
                    }
                    else
                    {
                        $result = ' `' . $this->dbo->escapeString($prefix) . '`.`intValue` IN ( ' . $this->dbo->mergeInClause($value) . ' ) ';
                    }
                }

                break;

            case BOL_QuestionService::QUESTION_PRESENTATION_DATE :
            case BOL_QuestionService::QUESTION_PRESENTATION_BIRTHDATE :
            case BOL_QuestionService::QUESTION_PRESENTATION_AGE :


                $value = explode('-', $value);
                $value = array('from' => $value[0], 'to' => $value[1]);

                if ( isset($value['from']) && isset($value['to']) )
                {
                    $maxDate = ( date('Y') - (int) $value['from'] ) . '-12-31';
                    $minDate = ( date('Y') - (int) $value['to'] ) . '-01-01';

                    if ( $question->base )
                    {
                        $result = " `user`.`" . $this->dbo->escapeString($question->name) . "` BETWEEN  '" . $this->dbo->escapeString($minDate) . "' AND '" . $this->dbo->escapeString($maxDate) . "'";
                    }
                    else
                    {
                        $result = " `" . $prefix . "`.`dateValue` BETWEEN  '" . $this->dbo->escapeString($minDate) . "' AND '" . $this->dbo->escapeString($maxDate) . "'";
                    }
                }

                break;

            case BOL_QuestionService::QUESTION_PRESENTATION_RANGE:
                $date = UTIL_DateTime::parseDate($value, UTIL_DateTime::MYSQL_DATETIME_DATE_FORMAT);
                $value = UTIL_DateTime::getAge($date['year'], $date['month'], $date['day']);
                if ( $question->base )
                {
                    $result = " " . $value . " BETWEEN SUBSTRING(`user`.`" . $this->dbo->escapeString($question->name) . "`, 1, LOCATE('-', `user`.`" . $this->dbo->escapeString($question->name) . "`)-1 ) AND SUBSTRING(`user`.`" . $this->dbo->escapeString($question->name) . "`, LOCATE('-', `user`.`" . $this->dbo->escapeString($question->name) . "`)+1 ) ";
                }
                else
                {
                    $result = " " . $value . " BETWEEN SUBSTRING(`" . $prefix . "`.`textValue`, 1, LOCATE('-', `" . $prefix . "`.`textValue`)-1 ) AND SUBSTRING(`" . $prefix . "`.`textValue`, LOCATE('-', `" . $prefix . "`.`textValue`)+1 )  ";
                }
                break;
        }

        return $result;
    }

    public function findRuleByQuestionName( $questionName )
    {
        $example = new OW_Example();
        $example->andFieldEqual('questionName', $questionName);

        return $this->findObjectByExample($example);
    }

    public function findMatchQuestionsForUser( $userId )
    {
        if ( $userId === null )
        {
            return array();
        }

        if (!OW::getPluginManager()->isPluginActive('oxdate'))
        {
            return array();
        }

        $genderAccTypes = OXDATE_BOL_AccountTypeToGenderService::getInstance()->findAll();
        $lookingForValue = BOL_QuestionService::getInstance()->getQuestionData(array($userId), array('match_sex'));
        $lookingForValues = array();
        foreach($genderAccTypes as $type)
        {
            if ($lookingForValue[$userId]['match_sex'] & $type->genderValue)
            {
                $lookingForValues[] = $type->genderValue;
            }
        }

        if (empty($lookingForValues))
        {
            return array();
        }

        $locationWhere = "";
        $locationJoin = "";
        $lookingForValues = $this->dbo->mergeInClause($lookingForValues);
        
        if ( OW::getPluginManager()->isPluginActive('googlelocation') )
        {
            $locationWhere = " OR  ( `question`.name = 'googlemap_location' AND `atg1`.`genderValue` IN ( {$lookingForValues} ) AND match.id IS NOT NULL ) ";
            $locationJoin = " LEFT JOIN " . BOL_QuestionToAccountTypeDao::getInstance()->getTableName() . " as `qta1` ON ( `question`.`name` = `qta1`.`questionName` )
                     LEFT JOIN `". OXDATE_BOL_AccountTypeToGenderDao::getInstance()->getTableName() ."` as `atg1` ON ( `qta1`.`accountType` = `atg1`.`accountType` )
                     LEFT JOIN `". MATCHMAKING_BOL_QuestionMatchDao::getInstance()->getTableName() ."` as `match` ON ( `match`.`questionName` = `question`.`name` ) ";
        }
        
        $sql = " SELECT DISTINCT `question`.* FROM `" . BOL_QuestionDao::getInstance()->getTableName() . "` as `question`

                    LEFT JOIN  `" . BOL_QuestionSectionDao::getInstance()->getTableName() . "` as `section`
                            ON ( `question`.`sectionName` = `section`.`name` AND `question`.`sectionName` != '' AND `question`.`sectionName` IS NOT NULL )

                    LEFT JOIN " . BOL_QuestionToAccountTypeDao::getInstance()->getTableName() . " as `qta` ON ( `question`.`parent` = `qta`.`questionName` )
                    LEFT JOIN `". OXDATE_BOL_AccountTypeToGenderDao::getInstance()->getTableName() ."` as `atg` ON ( `qta`.`accountType` = `atg`.`accountType` )
                        
                    {$locationJoin}
                    
                    WHERE ( `question`.`sectionName`='about_my_match' AND `atg`.`genderValue` IN ( {$lookingForValues} ) ) {$locationWhere} 

                    ORDER BY IF( `section`.`name` IS NULL, 0, 1 ),  `section`.`sortOrder`, `question`.`sortOrder` ";

        return $this->dbo->queryForList($sql);
    }

    public function findCompatibilityByUserIdList($userId, $userIdList, $first, $count, $sortOrder)
    {
        $params = $this->getQueryParamsForUserIdList($userId, $userIdList, $sortOrder);
        $query = $this->prepareQuerySelectForUserIdList($params);
        
        $list = $this->dbo->queryForList($query, array_merge(array('first' => $first, 'count' => $count)));
        $maxValue = $this->getMaxPercentValue();
        
        foreach($list as $key => $value)
        {
            $list[$key]['compatibility'] = ceil(($list[$key]['compatibility'] * 100)/$maxValue);
        }
        
        return $list;
    }
}
