<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/
class MATCHMAKING_CMP_UserViewSection extends OW_Component
{
    public function __construct( $section, $sectionQuestions, $data, $labels, $template = 'table', $hideSection = false, $additionalParams = array() )
    {
        parent::__construct();

        $user = BOL_UserService::getInstance()->findUserById($additionalParams['userId']);

        if (!isset($user))
        {
            $this->setVisible(false);
            return;
        }

        $sections = array();
        foreach($sectionQuestions as $question)
        {
            if ($question['parent'] == null) {
                $accTypeList = BOL_QuestionService::getInstance()->findAccountTypeListByQuestionName($question['name']);
            }
            else
            {
                $accTypeList = BOL_QuestionService::getInstance()->findAccountTypeListByQuestionName($question['parent']);
            }

            if (count($accTypeList) > 1)
            {
                if (!isset($sections['general']))
                {
                    $sections['general'] = array();
                    $sections['general']['name'] = 'general';
                    $sections['general']['label'] = OW::getLanguage()->text('matchmaking', "questions_general_label");
                    $sections['general']['questions'] = array();
                }

                $sections['general']['questions'][] = $question;
            }
            else
            {
                if (!isset($sections[$accTypeList[0]->accountType]))
                {
                    $sections[$accTypeList[0]->accountType] = array();
                    $sections[$accTypeList[0]->accountType]['name'] = $accTypeList[0]->accountType;
                    $sections[$accTypeList[0]->accountType]['label'] = BOL_QuestionService::getInstance()->getAccountTypeLang($accTypeList[0]->accountType);
                    $sections[$accTypeList[0]->accountType]['questions'] = array();
                }

                $sections[$accTypeList[0]->accountType]['questions'][] = $question;
            }
        }

        $this->assign('sections', $sections);

        $this->assign('sectionName', $section);
        $this->assign('questions', $sectionQuestions);
        $this->assign('generalQuestions', !empty($sections['general']['questions']) ? $sections['general']['questions'] : array() );
        $this->assign('questionsData', $data);
        $this->assign('labels', $labels);
        $this->assign('display', !$hideSection);


        $genderAccTypes = OXDATE_BOL_AccountTypeToGenderService::getInstance()->findAll();
        $lookingForValue = BOL_QuestionService::getInstance()->getQuestionData(array($user->getId()), array('match_sex'));

        $lookingForValues = array();
        foreach($genderAccTypes as $type)
        {
            if (!empty($lookingForValue[$user->getId()]['match_sex']) && $lookingForValue[$user->getId()]['match_sex'] & $type->genderValue)
            {
                if (!in_array($type->genderValue, $lookingForValues))
                {
                    $lookingForValues[] = $type->genderValue;
                }
            }
        }

        if (count($lookingForValues) > 1)
        {
            $mode = 'floatbox';
        }
        else
        {
            $mode = 'table';
        }
        $mode = 'floatbox';

        switch ($mode)
        {
            case 'floatbox':
                $this->setTemplate(OW::getPluginManager()->getPlugin('matchmaking')->getCmpViewDir() . 'user_view_section_floatbox.html' );
                break;
            case 'table':
                $this->setTemplate(OW::getPluginManager()->getPlugin('matchmaking')->getCmpViewDir() . 'user_view_section_table.html' );
                break;
        }

        $title = OW::getLanguage()->text('base', 'questions_section_about_my_match_label');
        $js = UTIL_JsGenerator::composeJsString(" $('#matchmakingViewMatchSectionBtn').click(function(){

            var matchmakingViewMatchSectionFloatbox = new OW_FloatBox({\$title: '{$title}', \$contents: $('#matchmakingViewMatchSectionFloatbox'), width: '550px'});

        }); ");

        OW::getDocument()->addOnloadScript($js);
    }
}