<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/


class PCGALLERY_CLASS_PhotoBridge
{
    /**
     * Class instance
     *
     * @var PCGALLERY_CLASS_PhotoBridge
     */
    private static $classInstance;

    /**
     * Returns class instance
     *
     * @return PCGALLERY_CLASS_PhotoBridge
     */
    public static function getInstance()
    {
        if ( !isset(self::$classInstance) )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    private $defaultPhotoAlbumName = 'Photos of me';

    public function __construct()
    {

    }

    public function isActive()
    {
        return OW::getPluginManager()->isPluginActive("photo");
    }

    public function getAlbumName()
    {
        $albumName = OW::getLanguage()->text('pcgallery', 'photo_album_name');

        return empty($albumName) ? $this->defaultPhotoAlbumName : $albumName;
    }

    public function getAlbum( $userId )
    {
        $album = OW::getEventManager()->call("photo.album_find", array(
            'albumTitle' => $this->getAlbumName(),
            'userId' => $userId
        ));

        return empty($album) ? null : $album;
    }

    public function getPhotos( $userId, $limit = null )
    {
        $params = array(
            'userId' => $userId,
            "entityType" => "user",
            "entityId" => $userId,
            "privacy" => null
        );

        if ( $limit !== null )
        {
            $params['offset'] = $limit[0];
            $params['limit'] = $limit[1];
        }

        $photos = OW::getEventManager()->call("photo.entity_photos_find", $params);

        if ( empty($photos) )
        {
            return array();
        }

        $out = array();
        foreach ( $photos as $photoId => $photo )
        {
            $out[] = array(
                'id' => $photoId,
                'src' => $photo['photoUrl']
            );
        }

        return $out;
    }
    
    public function getAlbumPhotos( $userId, $albumId, $limit = null )
    {
        $params = array(
            'albumId' => $albumId,
            'userId' => $userId,
            "privacy" => null
        );

        if ( $limit !== null )
        {
            $params['offset'] = $limit[0];
            $params['limit'] = $limit[1];
        }

        $photos = OW::getEventManager()->call('photo.album_photos_find', $params);

        if ( empty($photos) )
        {
            return array();
        }

        $out = array();
        foreach ( $photos as $photo )
        {
            $out[] = array(
                'id' => $photo["id"],
                'src' => $photo['photoUrl']
            );
        }

        return $out;
    }

    public function collectAlbumSuggest( BASE_CLASS_EventCollector $e )
    {
        $e->add(OW::getLanguage()->text('pcgallery', 'photo_album_name'));
    }

    public function initFloatbox()
    {
        OW::getEventManager()->call('photo.init_floatbox');
    }

    public function init()
    {
        if ( !$this->isActive() ) return;
        
        //OW::getEventManager()->bind("photo.suggest_default_album", array($this, 'collectAlbumSuggest'));
    }
}