<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class WINKS_MCMP_ConsoleItems extends OW_MobileComponent
{
    private $service;

    public function __construct( $limit )
    {
        parent::__construct();

        $this->service = WINKS_BOL_Service::getInstance();

        $language = OW::getLanguage();
        $avatarService = BOL_AvatarService::getInstance();

        $userId = OW::getUser()->getId();
        $activeModes = json_decode(OW::getConfig()->getValue('mailbox', 'active_modes'));

        $winks = $this->service->findWinkList($userId, 0, $limit, $activeModes);
        $mode = is_array($activeModes) ? (in_array('chat', $activeModes)) ? 'chat' : 'mail' : 'chat';

        $viewedIds = $items = array();

        foreach ( $winks as $wink )
        {
            $item = array();

            $avatar = $avatarService->getDataForUserAvatars(array($wink->getUserId()), true, true, true, false);
            $avatar = $avatar[$wink->getUserId()];
            $item['avatar'] = $avatar;
            $item['viewed'] = $wink->getViewed();

            if ( $wink->getUserId() == $userId )
            {
                if ( $wink->getStatus() == WINKS_BOL_WinksDao::STATUS_ACCEPT && $wink->getWinkback() )
                {
                    $userService = BOL_UserService::getInstance();

                    $item['string'] = $language->text('winks', 'console_wink_accept_item', array(
                        'userUrl' => $userService->getUserUrl($wink->getPartnerId()),
                        'displayName' => $userService->getDisplayName($wink->getPartnerId())
                    ));

                    if ( $mode == 'mail' || $wink->messageType == 'mail' )
                    {
                        $item['url'] = OW::getRouter()->urlForRoute('mailbox_mail_conversation', array(
                            'convId' => $wink->getConversationId()
                        ));
                    }
                    else
                    {
                        $item['url'] = OW::getRouter()->urlForRoute('mailbox_chat_conversation', array(
                            'userId' => $wink->getPartnerId()
                        ));
                    }
                }
                else
                {
                    continue;
                }
            }
            elseif ( $wink->getStatus() == WINKS_BOL_WinksDao::STATUS_ACCEPT )
            {
                $item['string'] = $language->text('winks', 'console_wink_wait_item', array(
                    'userUrl' => $avatar['url'],
                    'displayName' => $avatar['title']
                ));

                if ( $mode == 'mail' || $wink->messageType == 'mail' )
                {
                    $item['url'] = OW::getRouter()->urlForRoute('mailbox_mail_conversation', array(
                        'convId' => $wink->getConversationId()
                    ));
                }
                else
                {
                    $item['url'] = OW::getRouter()->urlForRoute('mailbox_chat_conversation', array(
                        'userId' => $wink->getUserId()
                    ));
                }
            }
            else
            {
                $item['string'] = OW::getLanguage()->text('winks', 'console_wink_wait_item', array(
                    'userUrl'=> $avatar['url'],
                    'displayName'=>$avatar['title']
                ));

                $item['toolbar'] = array(
                    'accept' => array(
                        'label' => $language->text('winks', 'accept_request'),
                        'onclick' => 'Winks.accept(' . $wink->getUserId() . ',' . $wink->getPartnerId() . ');',
                        'userId' => $wink->getUserId()
                    ),
                    'ignore' => array(
                        'label' => $language->text('winks', 'ignore_request'),
                        'onclick' => 'Winks.ignore(' . $wink->getUserId() . ',' . $wink->getPartnerId() . ');',
                        'userId' => $wink->getUserId()
                    )
                );
            }

            $items[] = $item;
            $viewedIds[] = $wink->getId();
        }

        $this->assign('items', $items);
        $this->service->markViewedByIds($viewedIds);
    }
}
